//
// SaveAsCommand.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Aug 23, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using Gtk;
using Pcw.PcwTextEditor.ExtensionModel;
using Pcw.PcwTextEditor.PcwDocument;

namespace Pcw.PcwTextEditor.Commands
{
	/// <summary>
	/// The Save As command.
	/// </summary>
	class SaveAsCommand : ICommand
	{
		private string id = "";		
		private MenuItem menu_item;
		private ToolItem tool_item;
		
		public string Id
		{
			set {id = value;}
			get{return id;}
		}				
		
		public MenuItem MenuObject 
		{
			set 
			{
				menu_item = value;
				menu_item.Sensitive = false;
			}
		}
		
		public ToolItem ToolObject 
		{
			set 
			{
				tool_item = value;
				tool_item.Sensitive = false;
			}
		}		
		
		public void Run ()
		{
			FileChooserDialog fc = new FileChooserDialog("Save As", TextEditor.MainWin, FileChooserAction.Save,
		                          "Cancel",ResponseType.Cancel,
		                          "Save",ResponseType.Accept);				
				
			if (fc.Run() != (int)ResponseType.Accept) {
				fc.Destroy();
				return;
			}
			
			string file = fc.Filename;
			fc.Destroy();
			
			System.IO.File.WriteAllText(file, TextEditor.MainWin.CurrDocument.ViewText.Buffer.Text);
						
			// Open a new file
			TextEditor.MainWin.AddDocument(file);
		}
		
		public void DocumentChanged()
		{	
		}

		public void OnLoadDocument ()
		{
			SetSensitive(true);			
		}

		public void OnSaveDocument ()
		{
		}

		public void OnCreateDocument ()
		{
		}
		
		public void OnCloseDocument ()
		{
			if (TextEditor.MainWin.Documents.Count == 0)
				SetSensitive(false);
		}
		
		public void OnChangeDocument ()
		{
		}
		
		public void OnSelectDocument ()
		{
		}
		
		private void SetSensitive(bool b) 
		{
			if (menu_item != null)
				menu_item.Sensitive = b;
			if (tool_item != null)
				tool_item.Sensitive = b;			
		}
	}
}

