//
// MementoCareTaker.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Sep 16, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using System.Collections.Generic;

namespace Pcw.PcwTextEditor.PcwDocument
{
	internal class MementoCaretaker
    {
        private Stack<MementoString> undo_stack = new Stack<MementoString>();
        private Stack<MementoString> redo_stack = new Stack<MementoString>();
		private string[] break_chars = new string[5] {" ", "_", "-", "\n", "\r"};

        internal MementoString getUndoMemento()
        {
            if (undo_stack.Count > 0)
            {
                redo_stack.Push(undo_stack.Peek());
                return undo_stack.Pop();
            }

			return null;
        }
		
        internal MementoString getRedoMemento()
        {
            if (redo_stack.Count > 0)
            {
                undo_stack.Push(redo_stack.Peek());
                return redo_stack.Pop();
            }

			return null;
        }
		
        internal void InsertMementoForUndoRedo(MementoString memento)
        {
			if (memento == null)
				return;
			
			// We ensure that undo for inserted text is manipulated as a whole word
			if (memento.OperationType == TextChangeType.InsertText && !IsBrakeWord(memento.Text) && 			    
			    memento.Text.Length == 1 && IsUndoPossible()) {								
				
				MementoString m = undo_stack.Peek();
				
				if (m.OperationType == TextChangeType.InsertText && !IsBrakeWord(m.Text) ) {
					//Add char to last memento and change Pos
					m.Text = m.Text + memento.Text;
					return;
				}
			}
			
			undo_stack.Push(memento);
			
        }
		
        internal bool IsUndoPossible()
        {
			if (undo_stack.Count > 0)
				return true;
			return false;
		}
		
        internal bool IsRedoPossible()
        {
            if (redo_stack.Count > 0)
				return true;
			return false;
        }
		
		internal void Clear()
		{
			undo_stack.Clear();
			redo_stack.Clear();
		}

		private bool IsBrakeWord(string s)
		{			
			foreach (string c in break_chars)
				if (c == s)
					return true;
			
			return false;
		}
    }
}

