//
// GotoLine.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Oct 4, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using Gtk;

namespace Pcw.Addins.Search
{
	public class GotoLine
	{
		private TextBuffer buffer;
		private GotoLineGui gui;
		
		public TextBuffer Buffer {
			set {
				buffer = value;
			}
		}		
		
		public GotoLine () : this(null)
		{
		}
		
		public GotoLine (TextBuffer textBuffer)
		{			
			buffer = textBuffer;
		}

		public bool GoLine (string line) 
		{
			int l;
			try 
			{
				l = Convert.ToInt32(line);
			}
			catch 
			{
				return false;
			}
			return GoLine(l);
		}
		
		public bool GoLine (int line) 
		{
			if (buffer == null) 
				throw new InvalidOperationException ("TextBuffer buffer must be set. Use 'Buffer' property to pass a buffer object.");							
			
			if (line < 1 || line > buffer.LineCount || buffer.LineCount == 0)
				return false;
			
			TextIter iter = buffer.GetIterAtLine(line - 1);
			buffer.PlaceCursor(iter);
			
			return true;
		}
		
		public void ShowGotoLine(Window parent)
		{
			if (GotoLineGui.IsShown)
				return;
			
			gui = new GotoLineGui (this, parent);
			
		}
		
		public void CloseGotoLine()
		{
			if (!GotoLineGui.IsShown)
				return;
			
			gui.Close();
			
		}			
	}
}

