//
// HighlightingAddin.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Oct 14, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using Pcw.PcwTextEditor;
using Pcw.PcwTextEditor.ExtensionModel;

namespace Pcw.Addins.Highlighting
{
	public class HighlightingAddin : EditorExtension
	{
		private Highlight hl;
		
		public override void Initialize ()
		{
			hl = new Highlight();
		}
		
		public override void OnLoadDocument ()
		{
				
			HighlightDefinition hlDef = hl.GetDefinitionForExtension(TextEditor.MainWin.CurrDocument.DocInfo.Extension);
			
			if (hlDef == null) 
				return;
			
			DateTime begin = DateTime.Now;
			
			Definition.HighlightBlocks blocks = new Definition.HighlightBlocks();
			
			hl.ApplyHighlight(TextEditor.MainWin.CurrDocument.ViewText.Buffer, hlDef, blocks);
			
			TextEditor.MainWin.CurrDocument.Data.Add("Pcw.Highlighting.Definition", hlDef);	
			TextEditor.MainWin.CurrDocument.Data.Add("Pcw.Highlighting.Blocks", blocks);			
			
			DateTime end = DateTime.Now;
			
			long elapsedTicks = end.Ticks - begin.Ticks;
			TimeSpan elapsedSpan = new TimeSpan(elapsedTicks);
			Console.WriteLine("Highlighting loading time: {0} seconds.", elapsedSpan.Seconds);
			Console.WriteLine("Highlighting loading time: {0} milliseconds.", elapsedSpan.Milliseconds);
		}
		
		public override void OnChangeDocument ()
		{	
			if (!TextEditor.MainWin.CurrDocument.Data.ContainsKey("Pcw.Highlighting.Definition"))
				return;
			
			HighlightDefinition hlDef = (HighlightDefinition) TextEditor.MainWin.CurrDocument.Data["Pcw.Highlighting.Definition"];			
			Definition.HighlightBlocks blocks = (Definition.HighlightBlocks) TextEditor.MainWin.CurrDocument.Data["Pcw.Highlighting.Blocks"];
			
			hl.ApplyHighlight(TextEditor.MainWin.CurrDocument.ViewText.Buffer, 
				TextEditor.MainWin.CurrDocument.ViewText.ChangedInfo, 
				hlDef, blocks);
			
		}
	}
}








