//
// Pcw.FindTextTest.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Sep 20, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using Gtk;
using NUnit.Framework;
using Pcw.Addins.Search;

namespace Pcw.NUnit2Test
{
	//[TestFixture]
	public class FindTextTest
	{
		[Test]
		public void FindText ()
		{
			Application.Init();
			Pcw.PcwTextEditor.TextEditor.InitAddins();
						
			TextBuffer buffer = new TextBuffer(new TextTagTable());
			FindText find;
			
			find = new FindText("yellow");
			
			int result = 0;
						
			InvalidOperationException exception = Assert.Throws<InvalidOperationException>(delegate { find.Find("");});
			Assert.AreEqual("TextBuffer buffer must be set. Use 'Buffer' property to pass a buffer object.", exception.Message);
			
			find.Buffer = buffer;
			
			find.Find("");
			Assert.AreEqual (0, result, "Test 1");
			Assert.AreEqual (0, find.Founds.Count, "Test 2");
			
			result = find.Find("ABC");
			Assert.AreEqual (0, result, "Test 3");
			
			buffer.Text = "En un lugar de la Mancha, de cuyo nombre no quiero acordarme, no ha mucho tiempo que vivía un hidalgo de los de lanza en astillero, adarga antigua, rocín flaco y galgo corredor.";

			result = find.Find("Mancha");
			Assert.AreEqual (1, result, "Test 4");
			Assert.AreEqual (1, find.Founds.Count, "Test 5");
			Assert.AreEqual ("Mancha", find.Founds[0].Text, "Test 6");
			Assert.AreEqual (18, find.Founds[0].Position, "Test 7");
			
			result = find.Find("an");
			Assert.AreEqual (3, result, "Test 8");

			result = find.Find("An");
			Assert.AreEqual (0, result, "Test 9");
			
			// Ignore case
			result = find.Find("An", true, false);
			Assert.AreEqual (3, result, "Test 10");
			
			// Using regex
			result = find.Find("h.d.lgo", false, true);
			Assert.AreEqual (1, result, "Test 11");
			Assert.AreEqual ("hidalgo", find.Founds[0].Text, "Test 12");
			Assert.AreEqual (94, find.Founds[0].Position, "Test 13");
			
			result = find.Find("h.D.lgo", false, true);
			Assert.AreEqual (0, result, "Test 14");
			
			result = find.Find("h.D.lgo", true, true);
			Assert.AreEqual (1, result, "Test 15");
			Assert.AreEqual ("hidalgo", find.Founds[0].Text, "Test 16");

			result = find.Find(".an.", false, true);
			Assert.AreEqual (3, result, "Test 17");
			Assert.AreEqual ("Manc", find.Founds[0].Text, "Test 18"); // Mancha
			Assert.AreEqual ("lanz", find.Founds[1].Text, "Test 19"); // lanza
			Assert.AreEqual (" ant", find.Founds[2].Text, "Test 20"); // antigua
			
			// Paragraph
			buffer.Text = "ABC DEF \n" + 
					"abc def \n" +
					"ABC DEF \n" +
					"abc def";
			
			result = find.Find("ABC");
			Assert.AreEqual (2, result, "Test 21");
			
			result = find.Find("ABC", true, false);
			Assert.AreEqual (4, result, "Test 22");
			Assert.AreEqual (0, find.Founds[0].Position, "Test 23");
			Assert.AreEqual (9, find.Founds[1].Position, "Test 24");
			Assert.AreEqual (27, find.Founds[3].Position, "Test 25");
			
			// Regex
			result = find.Find("D.F", false, true);
			Assert.AreEqual (2, result, "Test 26");
			
			result = find.Find("D.F", true, true);
			Assert.AreEqual (4, result, "Test 27");
			
			// REPLACE
			
			result = find.Replace("ABC", "CBA", false, false);
			Assert.AreEqual (2, result, "Test 28");
			Assert.AreEqual ("CBA DEF \nabc def \nCBA DEF \nabc def", buffer.Text, "Test 29");
			Assert.AreEqual ("CBA", find.Founds[0].Text, "Test 30");
			Assert.AreEqual ("CBA", find.Founds[1].Text, "Test 31");	
			Assert.AreEqual (18, find.Founds[1].Position, "Test 32");
			
			result = find.Replace("ABC", "CBA", true, false);
			Assert.AreEqual (2, result, "Test 33");
			Assert.AreEqual ("CBA DEF \nCBA def \nCBA DEF \nCBA def", buffer.Text, "Test 34");
			Assert.AreEqual ("CBA", find.Founds[0].Text, "Test 35");
			Assert.AreEqual ("CBA", find.Founds[1].Text, "Test 36");	
			Assert.AreEqual (27, find.Founds[1].Position, "Test 37");
			
			result = find.Replace("DEF", "fed", false, true);
			Assert.AreEqual (2, result, "Test 38");
			Assert.AreEqual ("CBA fed \nCBA def \nCBA fed \nCBA def", buffer.Text, "Test 39");
			Assert.AreEqual ("fed", find.Founds[0].Text, "Test 40");
			Assert.AreEqual ("fed", find.Founds[1].Text, "Test 41");
			Assert.AreEqual (22, find.Founds[1].Position, "Test 42");
		}
		
		[Test]
		public void GotoLineTest ()
		{
			Application.Init();
						
			TextBuffer buffer = new TextBuffer(new TextTagTable());
			
			GotoLine go = new GotoLine ();
						
			InvalidOperationException exception = Assert.Throws<InvalidOperationException>(delegate { go.GoLine(5);});
			Assert.AreEqual("TextBuffer buffer must be set. Use 'Buffer' property to pass a buffer object.", exception.Message);
			
			go.Buffer = buffer;
			
			Assert.AreEqual(false, go.GoLine(5), "No lines");
			
			buffer.Text = "ABC";
			
			Assert.AreEqual(true, go.GoLine(1), "True - one line");
			Assert.AreEqual(false, go.GoLine(5), "false - There is one line");

			buffer.Text = buffer.Text + " \n" + 
				"DEF" + " \n" + 
					"HIJ";
			
			Assert.AreEqual(true, go.GoLine(1), "True - first line");
			Assert.AreEqual(true, go.GoLine(2), "True - second line");
			Assert.AreEqual(true, go.GoLine(3), "True - third line");
			Assert.AreEqual(false, go.GoLine(4), "false - There are only 3 line");			
			
			Assert.AreEqual(false, go.GoLine("A"), "False - It is not a valid integer");
			Assert.AreEqual(true, go.GoLine("3"), "True - string could be converted to int");
		}
	}
}
