//
// Pcw.Addin.Highlighting.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Nov 11, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using System.IO;
using Gtk;
using NUnit.Framework;
using Pcw.PcwTextEditor.PcwDocument;
using Pcw.Addins.Highlighting;
using Pcw.Addins.Highlighting.Definition;

namespace Pcw.NUnit2Test
{
	public class HighlightTest
	{
		[Test]
		public void Highlight ()
		{
			Application.Init();
			Pcw.PcwTextEditor.TextEditor.InitAddins();
			
			DirectText dText = new DirectText("HgTest", GetTestFile(), "cs");			
			Document doc = new Document(new Notebook(), dText);
			
			Assert.AreEqual (dText.Text, doc.Text, "Test 1");
			
			Highlight hg = new Highlight(GetDefinition());
			Assert.AreEqual (2, hg.Defs.Count, "Test 2");
			
			HighlightDefinition def = hg.GetDefinitionForExtension(doc.DocInfo.Extension, GetSyntax());	
			HighlightBlocks blocks = new HighlightBlocks();
			
			hg.ApplyHighlight(doc.ViewText.Buffer, def, blocks);
			
			TextBuffer buffer = doc.ViewText.Buffer;
			
			Assert.AreEqual (true, dText.Text == buffer.Text, "Test 3");
			
			Assert.AreEqual (true, def.CaseSensitive, "Test 4");
			
			// Verify if tags have been applied correctly at loading
			Assert.AreEqual (true, VerifyTag(buffer, 0, 17, "Pcw.Highlight.Tag.comment.5"), "Test 5.1");
			Assert.AreEqual (true, VerifyTag(buffer, 18, 23, "Pcw.Highlight.Tag.reserved.1"), "Test 5.2");	// using
			Assert.AreEqual (true, VerifyTag(buffer, 31, 48, "Pcw.Highlight.Tag.comment.5"), "Test 5.3");	// line comment
			Assert.AreEqual (true, VerifyTag(buffer, 49, 66, "Pcw.Highlight.Tag.comment.5"), "Test 5.4");	// line comment
			Assert.AreEqual (true, VerifyTag(buffer, 67, 76, "Pcw.Highlight.Tag.reserved.1"), "Test 5.5");	// namespace
			Assert.AreEqual (true, VerifyTag(buffer, 91, 97, "Pcw.Highlight.Tag.reserved.1"), "Test 5.6");	// public
			Assert.AreEqual (true, VerifyTag(buffer, 98, 103, "Pcw.Highlight.Tag.reserved.1"), "Test 5.7");	// class
			Assert.AreEqual (true, VerifyTag(buffer, 167, 177, "Pcw.Highlight.Tag.quote.2"), "Test 5.8");	// "someText"
			Assert.AreEqual (true, VerifyTag(buffer, 193, 354, "Pcw.Highlight.Tag.comment.6"), "Test 5.9");	// Block comment /*...*/
			Assert.AreEqual (true, VerifyTag(buffer, 361, 368, "Pcw.Highlight.Tag.reserved.1"), "Test 5.10");// private
			Assert.AreEqual (true, VerifyTag(buffer, 369, 373, "Pcw.Highlight.Tag.reserved.1"), "Test 5.11");// void
			Assert.AreEqual (true, VerifyTag(buffer, 386, 392, "Pcw.Highlight.Tag.reserved.1"), "Test 5.12");// string

						
			// Modifiyng line test (inserting and deleting) . Not ToCloseToken type.
			
			AddText(hg, def, doc, 3, "(Inserted in line comment) ", blocks);
			Assert.AreEqual (true, VerifyTag(buffer, 0, 44, "Pcw.Highlight.Tag.comment.5"), "Test 6.1");
			
			Assert.AreNotEqual (dText.Text, doc.Text, "6.2"); 
			dText.Save();										
			Assert.AreEqual (dText.Text, doc.Text, "6.3");
			
			DeleteText(hg, def, doc, 3, 30, blocks);
			Assert.AreEqual (true, VerifyTag(buffer, 0, 17, "Pcw.Highlight.Tag.comment.5"), "Test 6.4");
			
			AddText(hg, def, doc, 177, @", """, blocks); // Open a string, tag must be to EOL
			Assert.AreEqual (true, VerifyTag(buffer, 179, 181, "Pcw.Highlight.Tag.quote.2"), "Test 6.5");
			
			AddText(hg, def, doc, 180, @"someText2""", blocks);
			//Assert.AreEqual (true, VerifyTag(buffer, 179, 190, "Pcw.Highlight.Tag.quote.2"), "Test 6.6");
			
			
			// Inserting one line . Not ToCloseToken type.
			
			//AddText(hg, def, doc, 18, @"// New comment line at line 2" + "\n", blocks);
			//Assert.AreEqual (true, VerifyTag(buffer, 18, 47, "Pcw.Highlight.Tag.comment.4"), "Test 6.7");
						
			
			// Inserting block lines . Not ToCloseToken type.
			
			//AddText(hg, def, doc, 567, "         // Coment in sub 1\n         // Coment in sub 2\n", blocks);
			//Assert.AreEqual (true, VerifyTag(buffer, 576, 622, "Pcw.Highlight.Tag.comment.4"), "Test 6.8");
						
			
			// Delete one line . Not ToCloseToken type.
			
			// Delete block lines . Not ToCloseToken type.
			
			// Starting a new block (Comment until the end)
		
			//AddText(hg, def, doc, 623, "/*\n   public x\n", blocks);
			//Assert.AreEqual (true, VerifyTag(buffer, 623, 653, "Pcw.Highlight.Tag.comment.5"), "Test 6.9");
			
			//DeleteText(hg, def, doc, 624, 625, blocks);
			//Assert.AreEqual (true, VerifyTag(buffer, 627, 633, "Pcw.Highlight.Tag.reserved.1"), "Test 6.10");
			
			
			// closing the block
			
			//AddText(hg, def, doc, 639, "*/\n");
			//Assert.AreEqual (true, VerifyTag(buffer, 623, 641, "Pcw.Highlight.Tag.comment.5"), "Test 6.10");
			
			// Modifiyng the key or closeToken
			
			
			
			Console.WriteLine("Show: \n********************************\n" + 
				buffer.Text + 
				"\n********************************");			
			
		}
		
		private StringReader GetDefinition()
		{
			string s = "";
			
			AddLine(ref s, @"<definitions>");
			AddLine(ref s, @"   <definition name=""csharp"" label=""C#"" file=""csharp.xml"" extensions=""cs"" />");
			AddLine(ref s, @"   <definition name=""xml"" label=""Xml"" file=""xml1.xml"" extensions=""xml, build, page, rdl, rdlc"" />");
			AddLine(ref s, @"</definitions>");
			
			return new StringReader(s);
			
		}

		private StringReader GetSyntax()
		{
			string s = "";
			
			AddLine(ref s, @"<highlightDefinition caseSensitive=""true"" >");
			AddLine(ref s, @"   <styles>");
			AddLine(ref s, @"      <style name=""reserved"" foreColor=""0,0,255"" backColor="""" bold=""false"" italic=""false"" />");
			AddLine(ref s, @"      <style name=""comment"" foreColor=""7,125,7"" backColor="""" bold=""false"" italic=""false"" />");
			AddLine(ref s, @"      <style name=""region"" foreColor=""255,0,0"" backColor="""" bold=""false"" italic=""false"" />");			
			AddLine(ref s, @"      <style name=""quote"" foreColor=""152,17,17"" backColor="""" bold=""false"" italic=""false"" />");
			AddLine(ref s, @"   </styles>");			
			AddLine(ref s, @"   <separators>");
			AddLine(ref s, @"      <separator value=""."" />");
			AddLine(ref s, @"      <separator value="","" />");
			AddLine(ref s, @"      <separator value=""-"" />");
			AddLine(ref s, @"      <separator value=""+"" />");
			AddLine(ref s, @"      <separator value=""="" />");
			AddLine(ref s, @"      <separator value=""("" />");
			AddLine(ref s, @"      <separator value="")"" />");
			AddLine(ref s, @"      <separator value=""["" />");
			AddLine(ref s, @"      <separator value=""]"" />");
			AddLine(ref s, @"      <separator value=""{"" />");
			AddLine(ref s, @"      <separator value=""}"" />");
			AddLine(ref s, @"      <separator value="";"" />");
			AddLine(ref s, @"      <separator value=""?"" />");
			AddLine(ref s, @"   </separators>");
			AddLine(ref s, @"   <descriptors>");
			AddLine(ref s, @"      <descriptor style=""reserved"" type=""WholeWord"" description="""" >");
			AddLine(ref s, @"         <tokens>");
			AddLine(ref s, @"            <token value=""using""/>");
			AddLine(ref s, @"            <token value=""namespace""/>");			
			AddLine(ref s, @"            <token value=""abstract""/>");	
			AddLine(ref s, @"            <token value=""public""/>");
			AddLine(ref s, @"            <token value=""private""/>");
			AddLine(ref s, @"            <token value=""class""/>");
			AddLine(ref s, @"            <token value=""void""/>");			
			AddLine(ref s, @"            <token value=""string""/>");
			AddLine(ref s, @"            <token value=""int""/>");
			AddLine(ref s, @"         </tokens>");
			AddLine(ref s, @"      </descriptor>");
			AddLine(ref s, @"      <descriptor style=""quote"" type=""ToCloseToken"" description="""">");			
			AddLine(ref s, @"         <tokens>");
			AddLine(ref s, @"            <token value=""&quot;"" closeToken=""&quot;"" escapeToken=""\"" multiLine=""False"" />");	
			AddLine(ref s, @"         </tokens>");
			AddLine(ref s, @"      </descriptor>");
			AddLine(ref s, @"      <descriptor style=""region"" type=""ToEOL"" description="""" >");
			AddLine(ref s, @"         <tokens>");
			AddLine(ref s, @"            <token value=""#region""/>");
			AddLine(ref s, @"            <token value=""#endregion""/>");
			AddLine(ref s, @"         </tokens>");
			AddLine(ref s, @"      </descriptor>");
			AddLine(ref s, @"      <descriptor style=""comment"" type=""ToEOL"" description=""Comment line"" >");
			AddLine(ref s, @"         <tokens>");
			AddLine(ref s, @"            <token value=""//""/>");
			AddLine(ref s, @"         </tokens>");
			AddLine(ref s, @"      </descriptor>");
			AddLine(ref s, @"      <descriptor style=""comment"" type=""ToCloseToken"" description=""Comment block"">");
			AddLine(ref s, @"         <tokens>");
			AddLine(ref s, @"            <token value=""/*"" closeToken=""*/"" multiLine=""True""/>");
			AddLine(ref s, @"         </tokens>");
			AddLine(ref s, @"      </descriptor>");
			AddLine(ref s, @"   </descriptors>");
			AddLine(ref s, @"</highlightDefinition>");
			
			return new StringReader(s);
			
		}		
		
		private string GetTestFile()
		{
			string s = "";
			
			AddLine(ref s, @"// Comment line 1"); 										// 0-17
			AddLine(ref s, @"using System");											// 18-30
			AddLine(ref s, @"// Comment line 2");										// 31-48
			AddLine(ref s, @"// Comment line 3");										// 49-66	
			AddLine(ref s, @"namespace Pcw.Test");										// 67-85
			AddLine(ref s, @"{");														// 86-87
			AddLine(ref s, @"   public class Test ()"); 								// 88-111
			AddLine(ref s, @"   {");													// 112-116
			AddLine(ref s, @"      public Test ()");									// 117-137
			AddLine(ref s, @"      {");													// 138-145
			AddLine(ref s, @"         DoSomething(""someText"")");						// 146-178
			AddLine(ref s, @"      }");													// 179-186
			AddLine(ref s, @"      /*");												// 187-195
			AddLine(ref s, @"         Block comment line 1");							// 196-225
			AddLine(ref s, @"         Block comment line 2");							// 226-255
			AddLine(ref s, @"         Block comment line 3");							// 256-285
			AddLine(ref s, @"         Block comment line 4");							// 286-315
			AddLine(ref s, @"         Block comment line 5");							// 316-345
			AddLine(ref s, @"      */");												// 346-354
			AddLine(ref s, @"      private void DoSomething(string someParameter)");	// 355-407
			AddLine(ref s, @"      {");
			AddLine(ref s, @"         int i = 0;");
			AddLine(ref s, @"         if (i == 0)");
			AddLine(ref s, @"            i = i + 99;");
			AddLine(ref s, @"         Console.WriteLine(someParameter);");			
			AddLine(ref s, @"      }");
			AddLine(ref s, @"   }");
			AddLine(ref s, @"}");
			
			return s;
		}
		
		private void AddLine(ref string s, string toAdd)
		{
			s = s + toAdd + "\n";
		}
		
		private bool VerifyTag(TextBuffer buffer, int startChar, int endChar, string tagName) 
		{
			TextIter sIter = buffer.StartIter;
			TextIter eIter = buffer.StartIter;
			
			sIter.ForwardChars(startChar);
			eIter.ForwardChars(endChar);
			
			TextTag tag = buffer.TagTable.Lookup(tagName);
			
			if (tag == null) {
				Console.WriteLine("**** tag==NULL for tagName: {0}", tagName);
				return false;
			}
			
			if (!sIter.BeginsTag(tag)) {
				Console.WriteLine("**** sIter.BeginsTag failed");
				return false;
			}
			if (!eIter.EndsTag(tag)) {
				Console.WriteLine("**** eIter.EndsTag failed");
				return false;
			}
			
			return true;
		}
		
		private void AddText(Highlight hg, HighlightDefinition def, Document doc, int start, string text, HighlightBlocks blocks)
		{
			TextIter iter = doc.ViewText.Buffer.StartIter;
			iter.ForwardChars(start);
			doc.ViewText.Buffer.Insert (ref iter, text);
			hg.ApplyHighlight(doc.ViewText.Buffer, doc.ViewText.ChangedInfo, def, blocks);
		}
		
		private void DeleteText(Highlight hg, HighlightDefinition def, Document doc, int start, int end, HighlightBlocks blocks)
		{
			TextIter startIter = doc.ViewText.Buffer.StartIter;
			TextIter endIter = doc.ViewText.Buffer.StartIter;
								
			startIter.ForwardChars(start);
			endIter.ForwardChars(end);
			
			doc.ViewText.Buffer.Delete(ref startIter, ref endIter);	
			hg.ApplyHighlight(doc.ViewText.Buffer, doc.ViewText.ChangedInfo, def, blocks);
		}		
	}
}

