//
// UndoRedoTest.cs
//
// Author: 
//  Fredy Ramirez <framirez@pacificcoolwave.com>
//
// Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)
//
//  Creation date: Sep 12, 2011
//  Last modification: 

/*
The MIT License

Copyright (C) 2011 Fredy Ramirez (http://www.pacificcoolwave.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using Pcw.PcwTextEditor;
using Gtk;
using NUnit.Framework;
using Pcw.PcwTextEditor.PcwDocument;

namespace Pcw.NUnit2Test
{
	//[TestFixture]
	public class UndoRedoTest
	{
		[Test]
		public void UndoRedo ()
		{
			Application.Init();
			Pcw.PcwTextEditor.TextEditor.InitAddins();
			
			Document doc = new Document(new Notebook(), new DirectText("Test", "", "txt"));
			
			Assert.AreEqual (doc.Text, "", "Buffer must initialize with no chars");
						
			doc.Text = "En un lugar de la mancha";
			
			doc.ViewText.Undo();
			Assert.AreEqual ("", doc.Text, "Undo failed");
					
			doc.Text = "En un lugar de la mancha";
			TextIter iter = doc.ViewText.Buffer.EndIter;
			doc.ViewText.Buffer.Insert (ref iter, " de cuyo nombre no quiero acordarme");
				
			doc.ViewText.Undo();
			Assert.AreEqual ("En un lugar de la mancha", doc.Text, "Undo failed");
						
			doc.ViewText.Redo();
			Assert.AreEqual ("En un lugar de la mancha de cuyo nombre no quiero acordarme", doc.Text, "Redo failed");			
			
			doc.ViewText.Undo();
			Assert.AreEqual ("En un lugar de la mancha", doc.Text, "Undo failed");
			
			doc.Text = "";
			Assert.AreEqual ("", doc.Text, "Delete failed");
			
			
			// Undo entire word
			
			iter = doc.ViewText.Buffer.StartIter;
			doc.ViewText.Buffer.Insert (ref iter, "f");
			doc.ViewText.Buffer.Insert (ref iter, "i");
			doc.ViewText.Buffer.Insert (ref iter, "r");
			doc.ViewText.Buffer.Insert (ref iter, "s");
			doc.ViewText.Buffer.Insert (ref iter, "t");
			doc.ViewText.Buffer.Insert (ref iter, " ");
			doc.ViewText.Buffer.Insert (ref iter, "s");
			doc.ViewText.Buffer.Insert (ref iter, "e");
			doc.ViewText.Buffer.Insert (ref iter, "c");
			doc.ViewText.Buffer.Insert (ref iter, "o");
			doc.ViewText.Buffer.Insert (ref iter, "n");
			doc.ViewText.Buffer.Insert (ref iter, "d");
			
			Assert.AreEqual ("first second", doc.Text, "doc.Text Failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("first ", doc.Text, "Undo failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("first", doc.Text, "Undo failed");
			doc.ViewText.Undo();
			Assert.AreEqual ("", doc.Text, "Undo failed");
			
			
			// Inserting text in the middle
			
			doc.ViewText.ClearUndoRedo();
			Assert.AreEqual (false, doc.ViewText.CanUndo, "ClearUndoRedo(CanUndo) failed");
			Assert.AreEqual (false, doc.ViewText.CanRedo, "ClearUndoRedo(CanRedo) failed");

			iter = doc.ViewText.Buffer.StartIter;
			doc.ViewText.Buffer.Insert (ref iter, "123");
			doc.ViewText.Buffer.Insert (ref iter, " ");
			doc.ViewText.Buffer.Insert (ref iter, "456");			
			Assert.AreEqual ("123 456", doc.Text, "Insert text failed");
			
			iter = doc.ViewText.Buffer.StartIter;
			iter.ForwardChars(3);
			doc.ViewText.Buffer.Insert (ref iter, " ");
			doc.ViewText.Buffer.Insert (ref iter, "XXX");
			Assert.AreEqual ("123 XXX 456", doc.Text, "Insert text failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("123  456", doc.Text, "Undo failed");

			doc.ViewText.Undo();
			Assert.AreEqual ("123 456", doc.Text, "Undo failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("123 ", doc.Text, "Undo failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("123", doc.Text, "Undo failed");
			
			doc.ViewText.Undo();
			Assert.AreEqual ("", doc.Text, "Undo failed");
			
			Assert.AreEqual (false, doc.ViewText.CanUndo, "CanUndo failed");
			
			doc.ViewText.Redo();
			Assert.AreEqual ("123", doc.Text, "Redo failed");
			
			doc.ViewText.Redo();
			Assert.AreEqual ("123 ", doc.Text, "Redo failed");
			
			doc.ViewText.Redo();
			Assert.AreEqual ("123 456", doc.Text, "Redo failed");
			
			doc.ViewText.Redo();
			Assert.AreEqual ("123  456", doc.Text, "Redo failed");
			
			doc.ViewText.Redo();
			Assert.AreEqual ("123 XXX 456", doc.Text, "Redo failed");
			
			Assert.AreEqual (false, doc.ViewText.CanRedo, "CanRedo failed");
			
		}
	}
}

